@@ -4,13 +4,12 @@ import tensorflow as tf
 
 
 def cnn(input_var,
-        filter_dims,
-        num_filters,
+        filters,
         strides,
         name,
         padding,
         hidden_nonlinearity=tf.nn.relu,
-        hidden_w_init=tf.glorot_uniform_initializer(),
+        hidden_w_init=tf.initializers.glorot_uniform(),
         hidden_b_init=tf.zeros_initializer()):
     """Convolutional neural network (CNN).
 
@@ -19,13 +18,11 @@ def cnn(input_var,
 
     Args:
         input_var (tf.Tensor): Input tf.Tensor to the CNN.
-        filter_dims (tuple[int]): Dimension of the filters. For example,
-            (3, 5) means there are two convolutional layers. The filter for
-            first layer is of dimension (3 x 3) and the second one is of
-            dimension (5 x 5).
-        num_filters (tuple[int]): Number of filters. For example, (3, 32) means
-            there are two convolutional layers. The filter for the first layer
-            has 3 channels and the second one with 32 channels.
+        filters (Tuple[Tuple[int, Tuple[int, int]], ...]): Number and dimension
+            of filters. For example, ((3, (3, 5)), (32, (3, 3))) means there
+            are two convolutional layers. The filter for the first layer have 3
+            channels and its shape is (3 x 5), while the filter for the second
+            layer have 32 channels and its shape is (3 x 3).
         strides (tuple[int]): The stride of the sliding window. For example,
             (1, 2) means there are two convolutional layers. The stride of the
             filter for first layer is 1 and that of the second layer is 2.
@@ -43,17 +40,15 @@ def cnn(input_var,
             tf.Tensor.
 
     Return:
-        The output tf.Tensor of the CNN.
+        tf.Tensor: The output tf.Tensor of the CNN.
 
     """
     with tf.compat.v1.variable_scope(name):
         h = input_var
-        for index, (filter_dim, num_filter,
-                    stride) in enumerate(zip(filter_dims, num_filters,
-                                             strides)):
+        for index, (filter_iter, stride) in enumerate(zip(filters, strides)):
             _stride = [1, stride, stride, 1]
-            h = _conv(h, 'h{}'.format(index), filter_dim, num_filter, _stride,
-                      hidden_w_init, hidden_b_init, padding)
+            h = _conv(h, 'h{}'.format(index), filter_iter[1], filter_iter[0],
+                      _stride, hidden_w_init, hidden_b_init, padding)
             if hidden_nonlinearity is not None:
                 h = hidden_nonlinearity(h)
 
@@ -63,15 +58,14 @@ def cnn(input_var,
 
 
 def cnn_with_max_pooling(input_var,
-                         filter_dims,
-                         num_filters,
+                         filters,
                          strides,
                          name,
                          pool_shapes,
                          pool_strides,
                          padding,
                          hidden_nonlinearity=tf.nn.relu,
-                         hidden_w_init=tf.glorot_uniform_initializer(),
+                         hidden_w_init=tf.initializers.glorot_uniform(),
                          hidden_b_init=tf.zeros_initializer()):
     """Convolutional neural network (CNN) with max-pooling.
 
@@ -80,13 +74,11 @@ def cnn_with_max_pooling(input_var,
 
     Args:
         input_var (tf.Tensor): Input tf.Tensor to the CNN.
-        filter_dims (tuple[int]): Dimension of the filters. For example,
-            (3, 5) means there are two convolutional layers. The filter for
-            first layer is of dimension (3 x 3) and the second one is of
-            dimension (5 x 5).
-        num_filters (tuple[int]): Number of filters. For example, (3, 32) means
-            there are two convolutional layers. The filter for the first layer
-            has 3 channels and the second one with 32 channels.
+        filters (Tuple[Tuple[int, Tuple[int, int]], ...]): Number and dimension
+            of filters. For example, ((3, (3, 5)), (32, (3, 3))) means there
+            are two convolutional layers. The filter for the first layer have 3
+            channels and its shape is (3 x 5), while the filter for the second
+            layer have 32 channels and its shape is (3 x 3).
         strides (tuple[int]): The stride of the sliding window. For example,
             (1, 2) means there are two convolutional layers. The stride of the
             filter for first layer is 1 and that of the second layer is 2.
@@ -110,7 +102,7 @@ def cnn_with_max_pooling(input_var,
             tf.Tensor.
 
     Return:
-        The output tf.Tensor of the CNN.
+        tf.Tensor: The output tf.Tensor of the CNN.
 
     """
     pool_strides = [1, pool_strides[0], pool_strides[1], 1]
@@ -118,12 +110,10 @@ def cnn_with_max_pooling(input_var,
 
     with tf.compat.v1.variable_scope(name):
         h = input_var
-        for index, (filter_dim, num_filter,
-                    stride) in enumerate(zip(filter_dims, num_filters,
-                                             strides)):
+        for index, (filter_iter, stride) in enumerate(zip(filters, strides)):
             _stride = [1, stride, stride, 1]
-            h = _conv(h, 'h{}'.format(index), filter_dim, num_filter, _stride,
-                      hidden_w_init, hidden_b_init, padding)
+            h = _conv(h, 'h{}'.format(index), filter_iter[1], filter_iter[0],
+                      _stride, hidden_w_init, hidden_b_init, padding)
             if hidden_nonlinearity is not None:
                 h = hidden_nonlinearity(h)
             h = tf.nn.max_pool2d(h,
@@ -138,10 +128,36 @@ def cnn_with_max_pooling(input_var,
 
 def _conv(input_var, name, filter_size, num_filter, strides, hidden_w_init,
           hidden_b_init, padding):
+    """Helper function for performing convolution.
+
+    Args:
+        input_var (tf.Tensor): Input tf.Tensor to the CNN.
+        name (str): Variable scope of the convolution Op.
+        filter_size (tuple[int]): Dimension of the filter. For example,
+            (3, 5) means the dimension of the filter is (3 x 5).
+        num_filter (int): Number of filters. For example, (3, 32) means
+            there are two convolutional layers. The filter for the first layer
+            has 3 channels and the second one with 32 channels.
+        strides (tuple[int]): The stride of the sliding window. For example,
+            (1, 2) means there are two convolutional layers. The stride of the
+            filter for first layer is 1 and that of the second layer is 2.
+        hidden_w_init (callable): Initializer function for the weight
+            of intermediate dense layer(s). The function should return a
+            tf.Tensor.
+        hidden_b_init (callable): Initializer function for the bias
+            of intermediate dense layer(s). The function should return a
+            tf.Tensor.
+        padding (str): The type of padding algorithm to use,
+            either 'SAME' or 'VALID'.
+
+    Return:
+        tf.Tensor: The output of the convolution.
+
+    """
     # channel from input
-    input_shape = input_var.get_shape()[-1].value
+    input_shape = input_var.get_shape()[-1]
     # [filter_height, filter_width, in_channels, out_channels]
-    w_shape = [filter_size, filter_size, input_shape, num_filter]
+    w_shape = [filter_size[0], filter_size[1], input_shape, num_filter]
     b_shape = [1, 1, 1, num_filter]
 
     with tf.compat.v1.variable_scope(name):
