@@ -1,15 +1,22 @@
-"""GaussianMLPModel."""
+"""Gaussian MLP Model.
+
+A model represented by a Gaussian distribution
+which is parameterized by a multilayer perceptron (MLP).
+"""
 import numpy as np
 import tensorflow as tf
+import tensorflow_probability as tfp
 
-from garage.tf.distributions import DiagonalGaussian
-from garage.tf.models.base import Model
 from garage.tf.models.mlp import mlp
+from garage.tf.models.model import Model
 from garage.tf.models.parameter import parameter
 
 
 class GaussianMLPModel(Model):
-    """GaussianMLPModel.
+    """Gaussian MLP Model.
+
+    A model represented by a Gaussian distribution
+    which is parameterized by a multilayer perceptron (MLP).
 
     Args:
         output_dim (int): Output dimension of the model.
@@ -48,8 +55,14 @@ class GaussianMLPModel(Model):
             to avoid numerical issues.
         max_std (float): If not None, the std is at most the value of max_std,
             to avoid numerical issues.
-        std_hidden_nonlinearity: Nonlinearity for each hidden layer in
-            the std network.
+        std_hidden_nonlinearity (callable): Nonlinearity for each hidden layer
+            in the std network.
+        std_hidden_w_init (callable): Initializer function for the weight
+            of intermediate dense layer(s) in the std network. The function
+            should return a tf.Tensor.
+        std_hidden_b_init (callable): Initializer function for the bias
+            of intermediate dense layer(s) in the std network. The function
+            should return a tf.Tensor.
         std_output_nonlinearity (callable): Activation function for output
             dense layer in the std network. It should return a tf.Tensor. Set
             it to None to maintain a linear activation.
@@ -61,6 +74,7 @@ class GaussianMLPModel(Model):
                exponential transformation
             - softplus: the std will be computed as log(1+exp(x))
         layer_normalization (bool): Bool for using layer normalization or not.
+
     """
 
     def __init__(self,
@@ -68,10 +82,10 @@ class GaussianMLPModel(Model):
                  name=None,
                  hidden_sizes=(32, 32),
                  hidden_nonlinearity=tf.nn.tanh,
-                 hidden_w_init=tf.glorot_uniform_initializer(),
+                 hidden_w_init=tf.initializers.glorot_uniform(),
                  hidden_b_init=tf.zeros_initializer(),
                  output_nonlinearity=None,
-                 output_w_init=tf.glorot_uniform_initializer(),
+                 output_w_init=tf.initializers.glorot_uniform(),
                  output_b_init=tf.zeros_initializer(),
                  learn_std=True,
                  adaptive_std=False,
@@ -81,10 +95,10 @@ class GaussianMLPModel(Model):
                  max_std=None,
                  std_hidden_sizes=(32, 32),
                  std_hidden_nonlinearity=tf.nn.tanh,
-                 std_hidden_w_init=tf.glorot_uniform_initializer(),
+                 std_hidden_w_init=tf.initializers.glorot_uniform(),
                  std_hidden_b_init=tf.zeros_initializer(),
                  std_output_nonlinearity=None,
-                 std_output_w_init=tf.glorot_uniform_initializer(),
+                 std_output_w_init=tf.initializers.glorot_uniform(),
                  std_parameterization='exp',
                  layer_normalization=False):
         # Network parameters
@@ -95,6 +109,7 @@ class GaussianMLPModel(Model):
         self._adaptive_std = adaptive_std
         self._std_share_network = std_share_network
         self._std_hidden_sizes = std_hidden_sizes
+        self._init_std = init_std
         self._min_std = min_std
         self._max_std = max_std
         self._std_hidden_nonlinearity = std_hidden_nonlinearity
@@ -110,11 +125,11 @@ class GaussianMLPModel(Model):
         self._output_w_init = output_w_init
         self._output_b_init = output_b_init
         self._layer_normalization = layer_normalization
-
         # Tranform std arguments to parameterized space
         self._init_std_param = None
         self._min_std_param = None
         self._max_std_param = None
+        # pylint: disable=assignment-from-no-return
         if self._std_parameterization == 'exp':
             self._init_std_param = np.log(init_std)
             if min_std is not None:
@@ -128,13 +143,36 @@ class GaussianMLPModel(Model):
             if max_std is not None:
                 self._max_std_param = np.log(np.exp(max_std) - 1)
         else:
-            raise NotImplementedError
+            raise ValueError("std parameterization should be or 'exp' or "
+                             "'softplus' but got {}".format(
+                                 self._std_parameterization))
 
     def network_output_spec(self):
-        """Network output spec."""
-        return ['mean', 'log_std', 'std_param', 'dist']
+        """Network output spec.
 
+        Return:
+            list[str]: List of key(str) for the network outputs.
+
+        """
+        return ['dist', 'mean', 'log_std']
+
+    # pylint: disable=arguments-differ
     def _build(self, state_input, name=None):
+        """Build model.
+
+        Args:
+            state_input (tf.Tensor): Entire time-series observation input.
+            name (str): Inner model name, also the variable scope of the
+                inner model, if exist. One example is
+                garage.tf.models.Sequential.
+
+        Returns:
+            tfp.distributions.MultivariateNormalDiag: Distribution.
+            tf.tensor: Mean.
+            tf.Tensor: Log of standard deviation.
+
+        """
+        del name
         action_dim = self._output_dim
 
         with tf.compat.v1.variable_scope('dist_params'):
@@ -201,6 +239,7 @@ class GaussianMLPModel(Model):
                             self._init_std_param),
                         trainable=self._learn_std,
                         name='log_std_network')
+                    log_std_network = tf.expand_dims(log_std_network, 1)
 
         mean_var = mean_network
         std_param = log_std_network
@@ -218,6 +257,6 @@ class GaussianMLPModel(Model):
             else:  # we know it must be softplus here
                 log_std_var = tf.math.log(tf.math.log(1. + tf.exp(std_param)))
 
-        dist = DiagonalGaussian(self._output_dim)
-
-        return mean_var, log_std_var, std_param, dist
+        return tfp.distributions.MultivariateNormalDiag(
+            loc=mean_var,
+            scale_diag=tf.exp(log_std_var)), mean_var, log_std_var
