@@ -1,4 +1,5 @@
 """Discrete CNN QFunction with CNN-MLP structure."""
+import akro
 import tensorflow as tf
 
 from garage.tf.models import CNNModel
@@ -6,7 +7,7 @@ from garage.tf.models import CNNModelWithMaxPooling
 from garage.tf.models import MLPDuelingModel
 from garage.tf.models import MLPModel
 from garage.tf.models import Sequential
-from garage.tf.q_functions import QFunction
+from garage.tf.q_functions.q_function import QFunction
 
 
 class DiscreteCNNQFunction(QFunction):
@@ -18,13 +19,11 @@ class DiscreteCNNQFunction(QFunction):
 
     Args:
         env_spec (garage.envs.env_spec.EnvSpec): Environment specification.
-        filter_dims (tuple[int]): Dimension of the filters. For example,
-            (3, 5) means there are two convolutional layers. The filter for
-            first layer is of dimension (3 x 3) and the second one is of
-            dimension (5 x 5).
-        num_filters (tuple[int]): Number of filters. For example, (3, 32) means
-            there are two convolutional layers. The filter for the first layer
-            has 3 channels and the second one with 32 channels.
+        filters (Tuple[Tuple[int, Tuple[int, int]], ...]): Number and dimension
+            of filters. For example, ((3, (3, 5)), (32, (3, 3))) means there
+            are two convolutional layers. The filter for the first layer have 3
+            channels and its shape is (3 x 5), while the filter for the second
+            layer have 32 channels and its shape is (3 x 3).
         strides (tuple[int]): The stride of the sliding window. For example,
             (1, 2) means there are two convolutional layers. The stride of the
             filter for first layer is 1 and that of the second layer is 2.
@@ -64,14 +63,14 @@ class DiscreteCNNQFunction(QFunction):
             a tf.Tensor.
         dueling (bool): Bool for using dueling network or not.
         layer_normalization (bool): Bool for using layer normalization or not.
+
     """
 
     def __init__(self,
                  env_spec,
-                 filter_dims,
-                 num_filters,
+                 filters,
                  strides,
-                 hidden_sizes=[256],
+                 hidden_sizes=(256, ),
                  name=None,
                  padding='SAME',
                  max_pooling=False,
@@ -79,18 +78,27 @@ class DiscreteCNNQFunction(QFunction):
                  pool_shapes=(2, 2),
                  cnn_hidden_nonlinearity=tf.nn.relu,
                  hidden_nonlinearity=tf.nn.relu,
-                 hidden_w_init=tf.glorot_uniform_initializer(),
+                 hidden_w_init=tf.initializers.glorot_uniform(),
                  hidden_b_init=tf.zeros_initializer(),
                  output_nonlinearity=None,
-                 output_w_init=tf.glorot_uniform_initializer(),
+                 output_w_init=tf.initializers.glorot_uniform(),
                  output_b_init=tf.zeros_initializer(),
                  dueling=False,
                  layer_normalization=False):
+        if not isinstance(env_spec.observation_space, akro.Box) or \
+                not len(env_spec.observation_space.shape) in (2, 3):
+            raise ValueError(
+                '{} can only process 2D, 3D akro.Image or'
+                ' akro.Box observations, but received an env_spec with '
+                'observation_space of type {} and shape {}'.format(
+                    type(self).__name__,
+                    type(env_spec.observation_space).__name__,
+                    env_spec.observation_space.shape))
+
         super().__init__(name)
         self._env_spec = env_spec
         self._action_dim = env_spec.action_space.n
-        self._filter_dims = filter_dims
-        self._num_filters = num_filters
+        self._filters = filters
         self._strides = strides
         self._hidden_sizes = hidden_sizes
         self._padding = padding
@@ -111,15 +119,13 @@ class DiscreteCNNQFunction(QFunction):
         action_dim = self._env_spec.action_space.flat_dim
 
         if not max_pooling:
-            cnn_model = CNNModel(filter_dims=filter_dims,
-                                 num_filters=num_filters,
+            cnn_model = CNNModel(filters=filters,
                                  strides=strides,
                                  padding=padding,
                                  hidden_nonlinearity=cnn_hidden_nonlinearity)
         else:
             cnn_model = CNNModelWithMaxPooling(
-                filter_dims=filter_dims,
-                num_filters=num_filters,
+                filters=filters,
                 strides=strides,
                 padding=padding,
                 pool_strides=pool_strides,
@@ -148,26 +154,50 @@ class DiscreteCNNQFunction(QFunction):
                 layer_normalization=layer_normalization)
 
         self.model = Sequential(cnn_model, output_model)
+        self._network = None
 
         self._initialize()
 
     def _initialize(self):
-        obs_ph = tf.compat.v1.placeholder(tf.float32, (None, ) + self.obs_dim,
-                                          name='obs')
+        """Initialize QFunction."""
+        if isinstance(self._env_spec.observation_space, akro.Image):
+            obs_ph = tf.compat.v1.placeholder(tf.uint8,
+                                              (None, ) + self.obs_dim,
+                                              name='obs')
+            augmented_obs_ph = tf.cast(obs_ph, tf.float32) / 255.0
+        else:
+            obs_ph = tf.compat.v1.placeholder(tf.float32,
+                                              (None, ) + self.obs_dim,
+                                              name='obs')
+            augmented_obs_ph = obs_ph
+
         with tf.compat.v1.variable_scope(self.name) as vs:
             self._variable_scope = vs
-            self.model.build(obs_ph)
+            self._network = self.model.build(augmented_obs_ph)
+
+        self._obs_input = obs_ph
 
     @property
     def q_vals(self):
-        """Q values."""
-        return self.model.networks['default'].outputs
+        """Return the Q values, the output of the network.
+
+        Return:
+            list[tf.Tensor]: Q values.
+
+        """
+        return self._network.outputs
 
     @property
     def input(self):
-        """Input tf.Tensor of the Q-function."""
-        return self.model.networks['default'].input
+        """Get input.
 
+        Return:
+            tf.Tensor: QFunction Input.
+
+        """
+        return self._obs_input
+
+    # pylint: disable=arguments-differ
     def get_qval_sym(self, state_input, name):
         """Symbolic graph for q-network.
 
@@ -176,10 +206,15 @@ class DiscreteCNNQFunction(QFunction):
             name (str): Network variable scope.
 
         Return:
-            The tf.Tensor output of Discrete CNN QFunction.
+            tf.Tensor: The tf.Tensor output of Discrete CNN QFunction.
+
         """
         with tf.compat.v1.variable_scope(self._variable_scope):
-            return self.model.build(state_input, name=name)
+            augmented_state_input = state_input
+            if isinstance(self._env_spec.observation_space, akro.Image):
+                augmented_state_input = tf.cast(state_input,
+                                                tf.float32) / 255.0
+            return self.model.build(augmented_state_input, name=name).outputs
 
     def clone(self, name):
         """Return a clone of the Q-function.
@@ -188,12 +223,15 @@ class DiscreteCNNQFunction(QFunction):
         not the parameters.
 
         Args:
-            name: Name of the newly created q-function.
+            name(str) : Name of the newly created q-function.
+
+        Returns:
+            garage.tf.q_functions.DiscreteCNNQFunction: Clone of this object
+
         """
         return self.__class__(name=name,
                               env_spec=self._env_spec,
-                              filter_dims=self._filter_dims,
-                              num_filters=self._num_filters,
+                              filters=self._filters,
                               strides=self._strides,
                               hidden_sizes=self._hidden_sizes,
                               padding=self._padding,
@@ -210,6 +248,23 @@ class DiscreteCNNQFunction(QFunction):
                               layer_normalization=self._layer_normalization)
 
     def __setstate__(self, state):
-        """Object.__setstate__."""
+        """Object.__setstate__.
+
+        Args:
+            state (dict): Unpickled state.
+
+        """
         self.__dict__.update(state)
         self._initialize()
+
+    def __getstate__(self):
+        """Object.__getstate__.
+
+        Returns:
+            dict: The state.
+
+        """
+        new_dict = self.__dict__.copy()
+        del new_dict['_obs_input']
+        del new_dict['_network']
+        return new_dict
