@@ -3,7 +3,7 @@ import tensorflow as tf
 
 from garage.tf.models import MLPDuelingModel
 from garage.tf.models import MLPModel
-from garage.tf.q_functions import QFunction
+from garage.tf.q_functions.q_function import QFunction
 
 
 class DiscreteMLPQFunction(QFunction):
@@ -36,7 +36,9 @@ class DiscreteMLPQFunction(QFunction):
         output_b_init (callable): Initializer function for the bias
             of output dense layer(s). The function should return a
             tf.Tensor.
+        dueling (bool): Use dueling model or not.
         layer_normalization (bool): Bool for using layer normalization.
+
     """
 
     def __init__(self,
@@ -44,10 +46,10 @@ class DiscreteMLPQFunction(QFunction):
                  name=None,
                  hidden_sizes=(32, 32),
                  hidden_nonlinearity=tf.nn.relu,
-                 hidden_w_init=tf.glorot_uniform_initializer(),
+                 hidden_w_init=tf.initializers.glorot_uniform(),
                  hidden_b_init=tf.zeros_initializer(),
                  output_nonlinearity=None,
-                 output_w_init=tf.glorot_uniform_initializer(),
+                 output_w_init=tf.initializers.glorot_uniform(),
                  output_b_init=tf.zeros_initializer(),
                  dueling=False,
                  layer_normalization=False):
@@ -89,26 +91,40 @@ class DiscreteMLPQFunction(QFunction):
                 output_b_init=output_b_init,
                 layer_normalization=layer_normalization)
 
+        self._network = None
+
         self._initialize()
 
     def _initialize(self):
+        """Initialize QFunction."""
         obs_ph = tf.compat.v1.placeholder(tf.float32, (None, ) + self.obs_dim,
                                           name='obs')
 
         with tf.compat.v1.variable_scope(self.name) as vs:
             self._variable_scope = vs
-            self.model.build(obs_ph)
+            self._network = self.model.build(obs_ph)
 
     @property
     def q_vals(self):
-        """Return the Q values, the output of the network."""
-        return self.model.networks['default'].outputs
+        """Return the Q values, the output of the network.
+
+        Return:
+            list[tf.Tensor]: Q values.
+
+        """
+        return self._network.outputs
 
     @property
     def input(self):
-        """Get input."""
-        return self.model.networks['default'].input
+        """Get input.
 
+        Return:
+            tf.Tensor: QFunction Input.
+
+        """
+        return self._network.input
+
+    # pylint: disable=arguments-differ
     def get_qval_sym(self, state_input, name):
         """Symbolic graph for q-network.
 
@@ -117,10 +133,11 @@ class DiscreteMLPQFunction(QFunction):
             name (str): Network variable scope.
 
         Return:
-            The tf.Tensor output of Discrete MLP QFunction.
+            tf.Tensor: The tf.Tensor output of Discrete MLP QFunction.
+
         """
         with tf.compat.v1.variable_scope(self._variable_scope):
-            return self.model.build(state_input, name=name)
+            return self.model.build(state_input, name=name).outputs
 
     def clone(self, name):
         """Return a clone of the Q-function.
@@ -130,6 +147,10 @@ class DiscreteMLPQFunction(QFunction):
 
         Args:
             name (str): Name of the newly created q-function.
+
+        Returns:
+            garage.tf.q_functions.DiscreteMLPQFunction: Clone of this object
+
         """
         return self.__class__(name=name,
                               env_spec=self._env_spec,
@@ -143,7 +164,23 @@ class DiscreteMLPQFunction(QFunction):
                               dueling=self._dueling,
                               layer_normalization=self._layer_normalization)
 
+    def __getstate__(self):
+        """Object.__getstate__.
+
+        Returns:
+            dict: The state.
+
+        """
+        new_dict = self.__dict__.copy()
+        del new_dict['_network']
+        return new_dict
+
     def __setstate__(self, state):
-        """Object.__setstate__."""
+        """Object.__setstate__.
+
+        Args:
+            state (dict): Unpickled state.
+
+        """
         self.__dict__.update(state)
         self._initialize()
