@@ -3,11 +3,12 @@ from dowel import tabular
 import numpy as np
 import tensorflow as tf
 
+from garage import make_optimizer
 from garage.tf.distributions import Bernoulli
 from garage.tf.misc import tensor_utils
 from garage.tf.models import NormalizedInputMLPModel
 from garage.tf.optimizers import ConjugateGradientOptimizer, LbfgsOptimizer
-from garage.tf.regressors import StochasticRegressor
+from garage.tf.regressors.regressor import StochasticRegressor
 
 
 class BernoulliMLPRegressor(StochasticRegressor):
@@ -61,10 +62,10 @@ class BernoulliMLPRegressor(StochasticRegressor):
                  name='BernoulliMLPRegressor',
                  hidden_sizes=(32, 32),
                  hidden_nonlinearity=tf.nn.relu,
-                 hidden_w_init=tf.glorot_uniform_initializer(),
+                 hidden_w_init=tf.initializers.glorot_uniform(),
                  hidden_b_init=tf.zeros_initializer(),
                  output_nonlinearity=tf.nn.sigmoid,
-                 output_w_init=tf.glorot_uniform_initializer(),
+                 output_w_init=tf.initializers.glorot_uniform(),
                  output_b_init=tf.zeros_initializer(),
                  optimizer=None,
                  optimizer_args=None,
@@ -82,23 +83,21 @@ class BernoulliMLPRegressor(StochasticRegressor):
 
         with tf.compat.v1.variable_scope(self._name, reuse=False) as vs:
             self._variable_scope = vs
-            if optimizer_args is None:
-                optimizer_args = dict()
-            if tr_optimizer_args is None:
-                tr_optimizer_args = dict()
+            optimizer_args = optimizer_args or dict()
+            tr_optimizer_args = tr_optimizer_args or dict()
 
             if optimizer is None:
-                optimizer = LbfgsOptimizer(**optimizer_args)
+                self._optimizer = make_optimizer(LbfgsOptimizer,
+                                                 **optimizer_args)
             else:
-                optimizer = optimizer(**optimizer_args)
+                self._optimizer = make_optimizer(optimizer, **optimizer_args)
 
             if tr_optimizer is None:
-                tr_optimizer = ConjugateGradientOptimizer(**tr_optimizer_args)
+                self._tr_optimizer = make_optimizer(ConjugateGradientOptimizer,
+                                                    **tr_optimizer_args)
             else:
-                tr_optimizer = tr_optimizer(**tr_optimizer_args)
-
-            self._optimizer = optimizer
-            self._tr_optimizer = tr_optimizer
+                self._tr_optimizer = make_optimizer(tr_optimizer,
+                                                    **tr_optimizer_args)
             self._first_optimized = False
 
         self.model = NormalizedInputMLPModel(
@@ -114,6 +113,7 @@ class BernoulliMLPRegressor(StochasticRegressor):
             layer_normalization=layer_normalization)
 
         self._dist = Bernoulli(output_dim)
+        self._network = None
 
         self._initialize()
 
@@ -123,7 +123,7 @@ class BernoulliMLPRegressor(StochasticRegressor):
                                              self._input_shape)
 
         with tf.compat.v1.variable_scope(self._variable_scope):
-            self.model.build(input_var)
+            self._network = self.model.build(input_var)
 
             ys_var = tf.compat.v1.placeholder(dtype=tf.float32,
                                               name='ys',
@@ -134,7 +134,7 @@ class BernoulliMLPRegressor(StochasticRegressor):
                                                     shape=(None,
                                                            self._output_dim))
 
-            y_hat = self.model.networks['default'].y_hat
+            y_hat = self._network.y_hat
 
             old_info_vars = dict(p=old_prob_var)
             info_vars = dict(p=y_hat)
@@ -170,10 +170,8 @@ class BernoulliMLPRegressor(StochasticRegressor):
         """
         if self._normalize_inputs:
             # recompute normalizing constants for inputs
-            self.model.networks['default'].x_mean.load(
-                np.mean(xs, axis=0, keepdims=True))
-            self.model.networks['default'].x_std.load(
-                np.std(xs, axis=0, keepdims=True) + 1e-8)
+            self._network.x_mean.load(np.mean(xs, axis=0, keepdims=True))
+            self._network.x_std.load(np.std(xs, axis=0, keepdims=True) + 1e-8)
 
         if self._use_trust_region and self._first_optimized:
             # To use trust region constraint and optimizer
@@ -245,15 +243,19 @@ class BernoulliMLPRegressor(StochasticRegressor):
 
         """
         with tf.compat.v1.variable_scope(self._variable_scope):
-            prob, _, _ = self.model.build(x_var, name=name)
+            prob, _, _ = self.model.build(x_var, name=name).outputs
 
         return self._dist.log_likelihood_sym(y_var, dict(p=prob))
 
-    def dist_info_sym(self, x_var, name=None):
+    # pylint: disable=unused-argument
+    def dist_info_sym(self, input_var, state_info_vars=None, name=None):
         """Build a symbolic graph of the distribution parameters.
 
         Args:
-            x_var (tf.Tensor): Input tf.Tensor for the input data.
+            input_var (tf.Tensor): Input tf.Tensor for the input data.
+            state_info_vars (dict): a dictionary whose values should contain
+                information about the state of the regressor at the time it
+                received the input.
             name (str): Name of the new graph.
 
         Return:
@@ -262,41 +264,43 @@ class BernoulliMLPRegressor(StochasticRegressor):
 
         """
         with tf.compat.v1.variable_scope(self._variable_scope):
-            prob, _, _ = self.model.build(x_var, name=name)
+            prob, _, _ = self.model.build(input_var, name=name).outputs
 
         return dict(prob=prob)
 
-    def get_params_internal(self, **args):
-        """Get the params, which are the trainable variables.
+    @property
+    def recurrent(self):
+        """bool: If this module has a hidden state."""
+        return False
 
-        Args:
-            args: Ignored by the function. Will be removed in future release.
+    @property
+    def vectorized(self):
+        """bool: If this module supports vectorization input."""
+        return True
 
-        Returns:
-            List[tf.Variable]: A list of trainable variables in the current
-            variable scope.
-
-        """
-        del args
-        return self._variable_scope.trainable_variables()
+    @property
+    def distribution(self):
+        """garage.tf.distributions.DiagonalGaussian: Distribution."""
+        return self._dist
 
     def __getstate__(self):
         """Object.__getstate__.
 
         Returns:
-            dict: the state to be pickled for the instance.
+            dict: The state to be pickled for the instance.
 
         """
         new_dict = super().__getstate__()
         del new_dict['_f_predict']
         del new_dict['_f_prob']
+        del new_dict['_network']
         return new_dict
 
     def __setstate__(self, state):
         """Object.__setstate__.
 
         Args:
-            state (dict): unpickled state.
+            state (dict): Unpickled state.
 
         """
         super().__setstate__(state)
