@@ -3,11 +3,12 @@ from dowel import tabular
 import numpy as np
 import tensorflow as tf
 
-from garage.tf.distributions import Categorical
+from garage import make_optimizer
 from garage.tf.misc import tensor_utils
-from garage.tf.models import NormalizedInputMLPModel
 from garage.tf.optimizers import ConjugateGradientOptimizer, LbfgsOptimizer
-from garage.tf.regressors import StochasticRegressor
+from garage.tf.regressors.categorical_mlp_regressor_model import (
+    CategoricalMLPRegressorModel)
+from garage.tf.regressors.regressor import StochasticRegressor
 
 
 class CategoricalMLPRegressor(StochasticRegressor):
@@ -65,10 +66,10 @@ class CategoricalMLPRegressor(StochasticRegressor):
                  name='CategoricalMLPRegressor',
                  hidden_sizes=(32, 32),
                  hidden_nonlinearity=tf.nn.tanh,
-                 hidden_w_init=tf.glorot_uniform_initializer(),
+                 hidden_w_init=tf.initializers.glorot_uniform(),
                  hidden_b_init=tf.zeros_initializer(),
                  output_nonlinearity=tf.nn.softmax,
-                 output_w_init=tf.glorot_uniform_initializer(),
+                 output_w_init=tf.initializers.glorot_uniform(),
                  output_b_init=tf.zeros_initializer(),
                  optimizer=None,
                  optimizer_args=None,
@@ -92,20 +93,20 @@ class CategoricalMLPRegressor(StochasticRegressor):
                 tr_optimizer_args = dict()
 
             if optimizer is None:
-                optimizer = LbfgsOptimizer(**optimizer_args)
+                self._optimizer = make_optimizer(LbfgsOptimizer,
+                                                 **optimizer_args)
             else:
-                optimizer = optimizer(**optimizer_args)
+                self._optimizer = make_optimizer(optimizer, **optimizer_args)
 
             if tr_optimizer is None:
-                tr_optimizer = ConjugateGradientOptimizer(**tr_optimizer_args)
+                self._tr_optimizer = make_optimizer(ConjugateGradientOptimizer,
+                                                    **tr_optimizer_args)
             else:
-                tr_optimizer = tr_optimizer(**tr_optimizer_args)
-
-            self._optimizer = optimizer
-            self._tr_optimizer = tr_optimizer
+                self._tr_optimizer = make_optimizer(tr_optimizer,
+                                                    **tr_optimizer_args)
             self._first_optimized = False
 
-        self.model = NormalizedInputMLPModel(
+        self.model = CategoricalMLPRegressorModel(
             input_shape,
             output_dim,
             hidden_sizes=hidden_sizes,
@@ -117,52 +118,50 @@ class CategoricalMLPRegressor(StochasticRegressor):
             output_b_init=output_b_init,
             layer_normalization=layer_normalization)
 
+        # model for old distribution, used when trusted region is on
+        self._old_model = self.model.clone(name='model_for_old_dist')
+        self._network = None
+        self._old_network = None
         self._initialize()
 
     def _initialize(self):
         input_var = tf.compat.v1.placeholder(tf.float32,
                                              shape=(None, ) +
                                              self._input_shape)
+        self._old_network = self._old_model.build(input_var)
 
         with tf.compat.v1.variable_scope(self._variable_scope):
-            self.model.build(input_var)
+            self._network = self.model.build(input_var)
+            self._old_model.parameters = self.model.parameters
 
             ys_var = tf.compat.v1.placeholder(dtype=tf.float32,
                                               name='ys',
                                               shape=(None, self._output_dim))
 
-            old_prob_var = tf.compat.v1.placeholder(dtype=tf.float32,
-                                                    name='old_prob',
-                                                    shape=(None,
-                                                           self._output_dim))
+            y_hat = self._network.y_hat
 
-            y_hat = self.model.networks['default'].y_hat
+            dist = self._network.dist
+            old_dist = self._old_network.dist
 
-            old_info_vars = dict(prob=old_prob_var)
-            info_vars = dict(prob=y_hat)
+            mean_kl = tf.reduce_mean(old_dist.kl_divergence(dist))
 
-            self._dist = Categorical(self._output_dim)
-            mean_kl = tf.reduce_mean(
-                self._dist.kl_sym(old_info_vars, info_vars))
-
-            loss = -tf.reduce_mean(
-                self._dist.log_likelihood_sym(ys_var, info_vars))
+            loss = -tf.reduce_mean(dist.log_prob(ys_var))
 
+            # pylint: disable=no-value-for-parameter
             predicted = tf.one_hot(tf.argmax(y_hat, axis=1),
                                    depth=self._output_dim)
 
             self._f_predict = tensor_utils.compile_function([input_var],
                                                             predicted)
-            self._f_prob = tensor_utils.compile_function([input_var], y_hat)
 
             self._optimizer.update_opt(loss=loss,
                                        target=self,
                                        inputs=[input_var, ys_var])
-            self._tr_optimizer.update_opt(
-                loss=loss,
-                target=self,
-                inputs=[input_var, ys_var, old_prob_var],
-                leq_constraint=(mean_kl, self._max_kl_step))
+            self._tr_optimizer.update_opt(loss=loss,
+                                          target=self,
+                                          inputs=[input_var, ys_var],
+                                          leq_constraint=(mean_kl,
+                                                          self._max_kl_step))
 
     def fit(self, xs, ys):
         """Fit with input data xs and label ys.
@@ -174,18 +173,16 @@ class CategoricalMLPRegressor(StochasticRegressor):
         """
         if self._normalize_inputs:
             # recompute normalizing constants for inputs
-            self.model.networks['default'].x_mean.load(
-                np.mean(xs, axis=0, keepdims=True))
-            self.model.networks['default'].x_std.load(
-                np.std(xs, axis=0, keepdims=True))
+            self._network.x_mean.load(np.mean(xs, axis=0, keepdims=True))
+            self._network.x_std.load(np.std(xs, axis=0, keepdims=True))
+            self._old_network.x_mean.load(np.mean(xs, axis=0, keepdims=True))
+            self._old_network.x_std.load(np.std(xs, axis=0, keepdims=True))
 
+        inputs = [xs, ys]
         if self._use_trust_region:
             # To use trust region constraint and optimizer
-            old_prob = self._f_prob(xs)
-            inputs = [xs, ys, old_prob]
             optimizer = self._tr_optimizer
         else:
-            inputs = [xs, ys]
             optimizer = self._optimizer
         loss_before = optimizer.loss(inputs)
         tabular.record('{}/LossBefore'.format(self._name), loss_before)
@@ -194,6 +191,7 @@ class CategoricalMLPRegressor(StochasticRegressor):
         tabular.record('{}/LossAfter'.format(self._name), loss_after)
         tabular.record('{}/dLoss'.format(self._name), loss_before - loss_after)
         self._first_optimized = True
+        self._old_model.parameters = self.model.parameters
 
     def predict(self, xs):
         """Predict ys based on input xs.
@@ -207,67 +205,20 @@ class CategoricalMLPRegressor(StochasticRegressor):
         """
         return self._f_predict(xs)
 
-    def predict_log_likelihood(self, xs, ys):
-        """Predict log-likelihood of output data conditioned on the input data.
+    @property
+    def recurrent(self):
+        """bool: If this module has a hidden state."""
+        return False
 
-        Args:
-            xs (numpy.ndarray): Input data.
-            ys (numpy.ndarray): Output labels in one hot representation.
+    @property
+    def vectorized(self):
+        """bool: If this module supports vectorization input."""
+        return True
 
-        Return:
-            numpy.ndarray: The predicted log likelihoods.
-
-        """
-        prob = self._f_prob(xs)
-        return self._dist.log_likelihood(ys, dict(prob=prob))
-
-    def dist_info_sym(self, x_var, name=None):
-        """Build a symbolic graph of the distribution parameters.
-
-        Args:
-            x_var (tf.Tensor): Input tf.Tensor for the input data.
-            name (str): Name of the new graph.
-
-        Return:
-            dict[tf.Tensor]: Output of the symbolic graph of the distribution
-                parameters.
-
-        """
-        with tf.compat.v1.variable_scope(self._variable_scope):
-            prob, _, _ = self.model.build(x_var, name=name)
-
-        return dict(prob=prob)
-
-    def log_likelihood_sym(self, x_var, y_var, name=None):
-        """Build a symbolic graph of the log-likelihood.
-
-        Args:
-            x_var (tf.Tensor): Input tf.Tensor for the input data.
-            y_var (tf.Tensor): Input tf.Tensor for the one hot label of data.
-            name (str): Name of the new graph.
-
-        Return:
-            tf.Tensor: Output of the symbolic log-likelihood graph.
-
-        """
-        with tf.compat.v1.variable_scope(self._variable_scope):
-            prob, _, _ = self.model.build(x_var, name=name)
-
-        return self._dist.log_likelihood_sym(y_var, dict(prob=prob))
-
-    def get_params_internal(self, **args):
-        """Get the params, which are the trainable variables.
-
-        Args:
-            args: Ignored by the function. Will be removed in future release.
-
-        Returns:
-            List[tf.Variable]: A list of trainable variables in the current
-            variable scope.
-
-        """
-        del args
-        return self._variable_scope.trainable_variables()
+    @property
+    def distribution(self):
+        """garage.tf.distributions.DiagonalGaussian: Distribution."""
+        return self._network.dist
 
     def __getstate__(self):
         """Object.__getstate__.
@@ -278,8 +229,8 @@ class CategoricalMLPRegressor(StochasticRegressor):
         """
         new_dict = super().__getstate__()
         del new_dict['_f_predict']
-        del new_dict['_f_prob']
-        del new_dict['_dist']
+        del new_dict['_network']
+        del new_dict['_old_network']
         return new_dict
 
     def __setstate__(self, state):
