@@ -3,10 +3,11 @@ from dowel import tabular
 import numpy as np
 import tensorflow as tf
 
+from garage import make_optimizer
 from garage.tf.misc import tensor_utils
 from garage.tf.models import NormalizedInputMLPModel
 from garage.tf.optimizers import LbfgsOptimizer
-from garage.tf.regressors import Regressor
+from garage.tf.regressors.regressor import Regressor
 
 
 class ContinuousMLPRegressor(Regressor):
@@ -51,10 +52,10 @@ class ContinuousMLPRegressor(Regressor):
                  name='ContinuousMLPRegressor',
                  hidden_sizes=(32, 32),
                  hidden_nonlinearity=tf.nn.tanh,
-                 hidden_w_init=tf.glorot_uniform_initializer(),
+                 hidden_w_init=tf.initializers.glorot_uniform(),
                  hidden_b_init=tf.zeros_initializer(),
                  output_nonlinearity=None,
-                 output_w_init=tf.glorot_uniform_initializer(),
+                 output_w_init=tf.initializers.glorot_uniform(),
                  output_b_init=tf.zeros_initializer(),
                  optimizer=None,
                  optimizer_args=None,
@@ -67,10 +68,10 @@ class ContinuousMLPRegressor(Regressor):
             if optimizer_args is None:
                 optimizer_args = dict()
             if optimizer is None:
-                optimizer = LbfgsOptimizer(**optimizer_args)
+                self._optimizer = make_optimizer(LbfgsOptimizer,
+                                                 **optimizer_args)
             else:
-                optimizer = optimizer(**optimizer_args)
-            self._optimizer = optimizer
+                self._optimizer = make_optimizer(optimizer, **optimizer_args)
 
         self.model = NormalizedInputMLPModel(
             input_shape=input_shape,
@@ -83,6 +84,8 @@ class ContinuousMLPRegressor(Regressor):
             output_w_init=output_w_init,
             output_b_init=output_b_init)
 
+        self._network = None
+
         self._initialize()
 
     def _initialize(self):
@@ -92,12 +95,12 @@ class ContinuousMLPRegressor(Regressor):
 
         with tf.compat.v1.variable_scope(self._name) as vs:
             self._variable_scope = vs
-            self.model.build(input_var)
+            self._network = self.model.build(input_var)
             ys_var = tf.compat.v1.placeholder(dtype=tf.float32,
                                               name='ys',
                                               shape=(None, self._output_dim))
 
-            y_hat = self.model.networks['default'].y_hat
+            y_hat = self._network.y_hat
             loss = tf.reduce_mean(tf.square(y_hat - ys_var))
 
             self._f_predict = tensor_utils.compile_function([input_var], y_hat)
@@ -122,10 +125,8 @@ class ContinuousMLPRegressor(Regressor):
         """
         if self._normalize_inputs:
             # recompute normalizing constants for inputs
-            self.model.networks['default'].x_mean.load(
-                np.mean(xs, axis=0, keepdims=True))
-            self.model.networks['default'].x_std.load(
-                np.std(xs, axis=0, keepdims=True) + 1e-8)
+            self._network.x_mean.load(np.mean(xs, axis=0, keepdims=True))
+            self._network.x_std.load(np.std(xs, axis=0, keepdims=True) + 1e-8)
 
         inputs = [xs, ys]
         loss_before = self._optimizer.loss(inputs)
@@ -159,22 +160,38 @@ class ContinuousMLPRegressor(Regressor):
 
         """
         with tf.compat.v1.variable_scope(self._variable_scope):
-            y_hat, _, _ = self.model.build(xs, name=name)
+            y_hat, _, _ = self.model.build(xs, name=name).outputs
 
         return y_hat
 
-    def get_params_internal(self, **args):
-        """Get the params, which are the trainable variables."""
-        del args
-        return self._variable_scope.trainable_variables()
+    @property
+    def recurrent(self):
+        """bool: If this module has a hidden state."""
+        return False
+
+    @property
+    def vectorized(self):
+        """bool: If this module supports vectorization input."""
+        return True
 
     def __getstate__(self):
-        """See `Object.__getstate__`."""
+        """Object.__getstate__.
+
+        Returns:
+            dict: the state to be pickled for the instance.
+
+        """
         new_dict = super().__getstate__()
         del new_dict['_f_predict']
+        del new_dict['_network']
         return new_dict
 
     def __setstate__(self, state):
-        """See `Object.__setstate__`."""
+        """Object.__setstate__.
+
+        Args:
+            state (dict): unpickled state.
+
+        """
         super().__setstate__(state)
         self._initialize()
