@@ -3,11 +3,12 @@ from dowel import tabular
 import numpy as np
 import tensorflow as tf
 
+from garage import make_optimizer
 from garage.tf.misc import tensor_utils
 from garage.tf.optimizers import LbfgsOptimizer, PenaltyLbfgsOptimizer
-from garage.tf.regressors import StochasticRegressor
 from garage.tf.regressors.gaussian_mlp_regressor_model import (
     GaussianMLPRegressorModel)
+from garage.tf.regressors.regressor import StochasticRegressor
 
 
 class GaussianMLPRegressor(StochasticRegressor):
@@ -69,10 +70,10 @@ class GaussianMLPRegressor(StochasticRegressor):
                  name='GaussianMLPRegressor',
                  hidden_sizes=(32, 32),
                  hidden_nonlinearity=tf.nn.tanh,
-                 hidden_w_init=tf.glorot_uniform_initializer(),
+                 hidden_w_init=tf.initializers.glorot_uniform(),
                  hidden_b_init=tf.zeros_initializer(),
                  output_nonlinearity=None,
-                 output_w_init=tf.glorot_uniform_initializer(),
+                 output_w_init=tf.initializers.glorot_uniform(),
                  output_b_init=tf.zeros_initializer(),
                  optimizer=None,
                  optimizer_args=None,
@@ -101,12 +102,13 @@ class GaussianMLPRegressor(StochasticRegressor):
                 optimizer_args = dict()
             if optimizer is None:
                 if use_trust_region:
-                    optimizer = PenaltyLbfgsOptimizer(**optimizer_args)
+                    self._optimizer = make_optimizer(PenaltyLbfgsOptimizer,
+                                                     **optimizer_args)
                 else:
-                    optimizer = LbfgsOptimizer(**optimizer_args)
+                    self._optimizer = make_optimizer(LbfgsOptimizer,
+                                                     **optimizer_args)
             else:
-                optimizer = optimizer(**optimizer_args)
-            self._optimizer = optimizer
+                self._optimizer = make_optimizer(optimizer, **optimizer_args)
 
         self.model = GaussianMLPRegressorModel(
             input_shape=input_shape,
@@ -130,60 +132,46 @@ class GaussianMLPRegressor(StochasticRegressor):
             std_parameterization='exp',
             layer_normalization=layer_normalization)
 
+        # model for old distribution, used when trusted region is on
+        self._old_model = self.model.clone(name='model_for_old_dist')
+        self._network = None
+        self._old_network = None
+
         self._initialize()
 
     def _initialize(self):
         input_var = tf.compat.v1.placeholder(tf.float32,
                                              shape=(None, ) +
                                              self._input_shape)
+        self._old_network = self._old_model.build(input_var)
 
         with tf.compat.v1.variable_scope(self._variable_scope):
-            self.model.build(input_var)
+            self._network = self.model.build(input_var)
+            self._old_model.parameters = self.model.parameters
+
             ys_var = tf.compat.v1.placeholder(dtype=tf.float32,
                                               name='ys',
                                               shape=(None, self._output_dim))
-            old_means_var = tf.compat.v1.placeholder(dtype=tf.float32,
-                                                     name='old_means',
-                                                     shape=(None,
-                                                            self._output_dim))
-            old_log_stds_var = tf.compat.v1.placeholder(
-                dtype=tf.float32,
-                name='old_log_stds',
-                shape=(None, self._output_dim))
 
-            y_mean_var = self.model.networks['default'].y_mean
-            y_std_var = self.model.networks['default'].y_std
-            means_var = self.model.networks['default'].means
-            log_stds_var = self.model.networks['default'].log_stds
-            normalized_means_var = self.model.networks[
-                'default'].normalized_means
-            normalized_log_stds_var = self.model.networks[
-                'default'].normalized_log_stds
+            y_mean_var = self._network.y_mean
+            y_std_var = self._network.y_std
+            means_var = self._network.mean
+
+            normalized_means_var = self._network.normalized_mean
+            normalized_log_stds_var = self._network.normalized_log_std
 
             normalized_ys_var = (ys_var - y_mean_var) / y_std_var
 
-            normalized_old_means_var = (old_means_var - y_mean_var) / y_std_var
-            normalized_old_log_stds_var = (old_log_stds_var -
-                                           tf.math.log(y_std_var))
-
-            normalized_dist_info_vars = dict(mean=normalized_means_var,
-                                             log_std=normalized_log_stds_var)
+            old_normalized_dist = self._old_network.normalized_dist
+            normalized_dist = self._network.normalized_dist
 
             mean_kl = tf.reduce_mean(
-                self.model.networks['default'].dist.kl_sym(
-                    dict(mean=normalized_old_means_var,
-                         log_std=normalized_old_log_stds_var),
-                    normalized_dist_info_vars,
-                ))
+                old_normalized_dist.kl_divergence(normalized_dist))
 
-            loss = -tf.reduce_mean(
-                self.model.networks['default'].dist.log_likelihood_sym(
-                    normalized_ys_var, normalized_dist_info_vars))
+            loss = -tf.reduce_mean(normalized_dist.log_prob(normalized_ys_var))
 
             self._f_predict = tensor_utils.compile_function([input_var],
                                                             means_var)
-            self._f_pdists = tensor_utils.compile_function(
-                [input_var], [means_var, log_stds_var])
 
             optimizer_args = dict(
                 loss=loss,
@@ -195,11 +183,7 @@ class GaussianMLPRegressor(StochasticRegressor):
 
             if self._use_trust_region:
                 optimizer_args['leq_constraint'] = (mean_kl, self._max_kl_step)
-                optimizer_args['inputs'] = [
-                    input_var, ys_var, old_means_var, old_log_stds_var
-                ]
-            else:
-                optimizer_args['inputs'] = [input_var, ys_var]
+            optimizer_args['inputs'] = [input_var, ys_var]
 
             with tf.name_scope('update_opt'):
                 self._optimizer.update_opt(**optimizer_args)
@@ -221,21 +205,19 @@ class GaussianMLPRegressor(StochasticRegressor):
 
         if self._normalize_inputs:
             # recompute normalizing constants for inputs
-            self.model.networks['default'].x_mean.load(
-                np.mean(xs, axis=0, keepdims=True))
-            self.model.networks['default'].x_std.load(
+            self._network.x_mean.load(np.mean(xs, axis=0, keepdims=True))
+            self._network.x_std.load(np.std(xs, axis=0, keepdims=True) + 1e-8)
+            self._old_network.x_mean.load(np.mean(xs, axis=0, keepdims=True))
+            self._old_network.x_std.load(
                 np.std(xs, axis=0, keepdims=True) + 1e-8)
         if self._normalize_outputs:
             # recompute normalizing constants for outputs
-            self.model.networks['default'].y_mean.load(
-                np.mean(ys, axis=0, keepdims=True))
-            self.model.networks['default'].y_std.load(
+            self._network.y_mean.load(np.mean(ys, axis=0, keepdims=True))
+            self._network.y_std.load(np.std(ys, axis=0, keepdims=True) + 1e-8)
+            self._old_network.y_mean.load(np.mean(ys, axis=0, keepdims=True))
+            self._old_network.y_std.load(
                 np.std(ys, axis=0, keepdims=True) + 1e-8)
-        if self._use_trust_region:
-            old_means, old_log_stds = self._f_pdists(xs)
-            inputs = [xs, ys, old_means, old_log_stds]
-        else:
-            inputs = [xs, ys]
+        inputs = [xs, ys]
         loss_before = self._optimizer.loss(inputs)
         tabular.record('{}/LossBefore'.format(self._name), loss_before)
         self._optimizer.optimize(inputs)
@@ -245,6 +227,7 @@ class GaussianMLPRegressor(StochasticRegressor):
             tabular.record('{}/MeanKL'.format(self._name),
                            self._optimizer.constraint_val(inputs))
         tabular.record('{}/dLoss'.format(self._name), loss_before - loss_after)
+        self._old_model.parameters = self.model.parameters
 
     def predict(self, xs):
         """Predict ys based on input xs.
@@ -258,58 +241,35 @@ class GaussianMLPRegressor(StochasticRegressor):
         """
         return self._f_predict(xs)
 
-    def log_likelihood_sym(self, x_var, y_var, name=None):
-        """Create a symbolic graph of the log likelihood.
+    @property
+    def vectorized(self):
+        """bool: If this module supports vectorization input."""
+        return True
 
-        Args:
-            x_var (tf.Tensor): Input tf.Tensor for the input data.
-            y_var (tf.Tensor): Input tf.Tensor for the label of data.
-            name (str): Name of the new graph.
-
-        Return:
-            tf.Tensor: Output of the symbolic log-likelihood graph.
-
-        """
-        params = self.dist_info_sym(x_var, name=name)
-        means_var = params['mean']
-        log_stds_var = params['log_std']
-
-        return self.model.networks[name].dist.log_likelihood_sym(
-            y_var, dict(mean=means_var, log_std=log_stds_var))
-
-    def dist_info_sym(self, x_var, name=None):
-        """Create a symbolic graph of the distribution parameters.
-
-        Args:
-            x_var (tf.Tensor): tf.Tensor of the input data.
-            name (str): Name of the new graph.
-
-        Return:
-            dict[tf.Tensor]: Outputs of the symbolic distribution parameter
-                graph.
-
-        """
-        with tf.compat.v1.variable_scope(self._variable_scope):
-            self.model.build(x_var, name=name)
-
-        means_var = self.model.networks[name].means
-        log_stds_var = self.model.networks[name].log_stds
-
-        return dict(mean=means_var, log_std=log_stds_var)
-
-    def get_params_internal(self, **args):
-        """Get the params, which are the trainable variables."""
-        del args
-        return self._variable_scope.trainable_variables()
+    @property
+    def distribution(self):
+        """garage.tf.distributions.DiagonalGaussian: Distribution."""
+        return self._network.dist
 
     def __getstate__(self):
-        """See `Object.__getstate__`."""
+        """Object.__getstate__.
+
+        Returns:
+            dict: The state to be pickled for the instance.
+
+        """
         new_dict = super().__getstate__()
         del new_dict['_f_predict']
-        del new_dict['_f_pdists']
+        del new_dict['_network']
+        del new_dict['_old_network']
         return new_dict
 
     def __setstate__(self, state):
-        """See `Object.__setstate__`."""
+        """Object.__setstate__.
+
+        Args:
+            state (dict): Unpickled state.
+
+        """
         super().__setstate__(state)
         self._initialize()
