@@ -2,7 +2,6 @@ import collections
 from copy import copy
 import pickle
 
-import dm_control.mujoco
 import dm_control.suite
 import pytest
 
@@ -10,6 +9,7 @@ from garage.envs.dm_control import DmControlEnv
 from tests.helpers import step_env
 
 
+@pytest.mark.mujoco
 class TestDmControlEnv:
 
     def test_can_step(self):
@@ -22,9 +22,7 @@ class TestDmControlEnv:
         a = act_space.sample()
         assert act_space.contains(a)
         # Skip rendering because it causes TravisCI to run out of memory
-        # Sometimes random actions lead to physics errors
-        with env._env.physics.suppress_physics_errors():
-            step_env(env, render=False)
+        step_env(env, render=False)
         env.close()
 
     @pytest.mark.nightly
@@ -39,9 +37,7 @@ class TestDmControlEnv:
         a = act_space.sample()
         assert act_space.contains(a)
         # Skip rendering because it causes TravisCI to run out of memory
-        # Sometimes random actions lead to physics errors
-        with env._env.physics.suppress_physics_errors():
-            step_env(env, render=False)
+        step_env(env, render=False)
         env.close()
 
     def test_pickleable(self):
@@ -50,9 +46,7 @@ class TestDmControlEnv:
         round_trip = pickle.loads(pickle.dumps(env))
         assert round_trip
         # Skip rendering because it causes TravisCI to run out of memory
-        # Sometimes random actions lead to physics errors
-        with env._env.physics.suppress_physics_errors():
-            step_env(env, render=False)
+        step_env(round_trip, render=False)
         round_trip.close()
         env.close()
 
@@ -64,9 +58,7 @@ class TestDmControlEnv:
         round_trip = pickle.loads(pickle.dumps(env))
         assert round_trip
         # Skip rendering because it causes TravisCI to run out of memory
-        # Sometimes random actions lead to physics errors
-        with env._env.physics.suppress_physics_errors():
-            step_env(env, render=False)
+        step_env(round_trip, render=False)
         round_trip.close()
         env.close()
 
