@@ -9,11 +9,11 @@ import numpy as np
 import pytest
 import tensorflow as tf
 
-from garage.np.exploration_strategies import EpsilonGreedyStrategy
-from garage.replay_buffer import SimpleReplayBuffer
+from garage.envs import GarageEnv
+from garage.experiment import LocalTFRunner
+from garage.np.exploration_policies import EpsilonGreedyPolicy
+from garage.replay_buffer import PathBuffer
 from garage.tf.algos import DQN
-from garage.tf.envs import TfEnv
-from garage.tf.experiment import LocalTFRunner
 from garage.tf.policies import DiscreteQfDerivedPolicy
 from garage.tf.q_functions import DiscreteMLPQFunction
 from tests.fixtures import snapshot_config, TfGraphTestCase
@@ -26,17 +26,16 @@ class TestDQN(TfGraphTestCase):
         """Test DQN with CartPole environment."""
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
             n_epochs = 10
-            n_epoch_cycles = 10
+            steps_per_epoch = 10
             sampler_batch_size = 500
-            num_timesteps = n_epochs * n_epoch_cycles * sampler_batch_size
-            env = TfEnv(gym.make('CartPole-v0'))
-            replay_buffer = SimpleReplayBuffer(env_spec=env.spec,
-                                               size_in_transitions=int(1e4),
-                                               time_horizon=1)
+            num_timesteps = n_epochs * steps_per_epoch * sampler_batch_size
+            env = GarageEnv(gym.make('CartPole-v0'))
+            replay_buffer = PathBuffer(capacity_in_transitions=int(1e4))
             qf = DiscreteMLPQFunction(env_spec=env.spec, hidden_sizes=(64, 64))
             policy = DiscreteQfDerivedPolicy(env_spec=env.spec, qf=qf)
-            epilson_greedy_strategy = EpsilonGreedyStrategy(
+            epilson_greedy_policy = EpsilonGreedyPolicy(
                 env_spec=env.spec,
+                policy=policy,
                 total_timesteps=num_timesteps,
                 max_epsilon=1.0,
                 min_epsilon=0.02,
@@ -44,20 +43,19 @@ class TestDQN(TfGraphTestCase):
             algo = DQN(env_spec=env.spec,
                        policy=policy,
                        qf=qf,
-                       exploration_strategy=epilson_greedy_strategy,
+                       exploration_policy=epilson_greedy_policy,
                        replay_buffer=replay_buffer,
                        qf_lr=1e-4,
                        discount=1.0,
                        min_buffer_size=int(1e3),
                        double_q=False,
                        n_train_steps=500,
-                       n_epoch_cycles=n_epoch_cycles,
+                       steps_per_epoch=steps_per_epoch,
                        target_network_update_freq=1,
                        buffer_batch_size=32)
 
             runner.setup(algo, env)
             last_avg_ret = runner.train(n_epochs=n_epochs,
-                                        n_epoch_cycles=n_epoch_cycles,
                                         batch_size=sampler_batch_size)
             assert last_avg_ret > 15
 
@@ -68,17 +66,16 @@ class TestDQN(TfGraphTestCase):
         """Test DQN with CartPole environment."""
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
             n_epochs = 10
-            n_epoch_cycles = 10
+            steps_per_epoch = 10
             sampler_batch_size = 500
-            num_timesteps = n_epochs * n_epoch_cycles * sampler_batch_size
-            env = TfEnv(gym.make('CartPole-v0'))
-            replay_buffer = SimpleReplayBuffer(env_spec=env.spec,
-                                               size_in_transitions=int(1e4),
-                                               time_horizon=1)
+            num_timesteps = n_epochs * steps_per_epoch * sampler_batch_size
+            env = GarageEnv(gym.make('CartPole-v0'))
+            replay_buffer = PathBuffer(capacity_in_transitions=int(1e4))
             qf = DiscreteMLPQFunction(env_spec=env.spec, hidden_sizes=(64, 64))
             policy = DiscreteQfDerivedPolicy(env_spec=env.spec, qf=qf)
-            epilson_greedy_strategy = EpsilonGreedyStrategy(
+            epilson_greedy_policy = EpsilonGreedyPolicy(
                 env_spec=env.spec,
+                policy=policy,
                 total_timesteps=num_timesteps,
                 max_epsilon=1.0,
                 min_epsilon=0.02,
@@ -86,20 +83,19 @@ class TestDQN(TfGraphTestCase):
             algo = DQN(env_spec=env.spec,
                        policy=policy,
                        qf=qf,
-                       exploration_strategy=epilson_greedy_strategy,
+                       exploration_policy=epilson_greedy_policy,
                        replay_buffer=replay_buffer,
                        qf_lr=1e-4,
                        discount=1.0,
                        min_buffer_size=int(1e3),
                        double_q=True,
                        n_train_steps=500,
-                       n_epoch_cycles=n_epoch_cycles,
+                       steps_per_epoch=steps_per_epoch,
                        target_network_update_freq=1,
                        buffer_batch_size=32)
 
             runner.setup(algo, env)
             last_avg_ret = runner.train(n_epochs=n_epochs,
-                                        n_epoch_cycles=n_epoch_cycles,
                                         batch_size=sampler_batch_size)
             assert last_avg_ret > 15
 
@@ -110,17 +106,16 @@ class TestDQN(TfGraphTestCase):
         """Test DQN with CartPole environment."""
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
             n_epochs = 10
-            n_epoch_cycles = 10
+            steps_per_epoch = 10
             sampler_batch_size = 500
-            num_timesteps = n_epochs * n_epoch_cycles * sampler_batch_size
-            env = TfEnv(gym.make('CartPole-v0'))
-            replay_buffer = SimpleReplayBuffer(env_spec=env.spec,
-                                               size_in_transitions=int(1e4),
-                                               time_horizon=1)
+            num_timesteps = n_epochs * steps_per_epoch * sampler_batch_size
+            env = GarageEnv(gym.make('CartPole-v0'))
+            replay_buffer = PathBuffer(capacity_in_transitions=int(1e4))
             qf = DiscreteMLPQFunction(env_spec=env.spec, hidden_sizes=(64, 64))
             policy = DiscreteQfDerivedPolicy(env_spec=env.spec, qf=qf)
-            epilson_greedy_strategy = EpsilonGreedyStrategy(
+            epilson_greedy_policy = EpsilonGreedyPolicy(
                 env_spec=env.spec,
+                policy=policy,
                 total_timesteps=num_timesteps,
                 max_epsilon=1.0,
                 min_epsilon=0.02,
@@ -128,7 +123,7 @@ class TestDQN(TfGraphTestCase):
             algo = DQN(env_spec=env.spec,
                        policy=policy,
                        qf=qf,
-                       exploration_strategy=epilson_greedy_strategy,
+                       exploration_policy=epilson_greedy_policy,
                        replay_buffer=replay_buffer,
                        qf_lr=1e-4,
                        discount=1.0,
@@ -136,13 +131,12 @@ class TestDQN(TfGraphTestCase):
                        double_q=False,
                        n_train_steps=500,
                        grad_norm_clipping=5.0,
-                       n_epoch_cycles=n_epoch_cycles,
+                       steps_per_epoch=steps_per_epoch,
                        target_network_update_freq=1,
                        buffer_batch_size=32)
 
             runner.setup(algo, env)
             last_avg_ret = runner.train(n_epochs=n_epochs,
-                                        n_epoch_cycles=n_epoch_cycles,
                                         batch_size=sampler_batch_size)
             assert last_avg_ret > 15
 
@@ -152,17 +146,16 @@ class TestDQN(TfGraphTestCase):
         """Test DQN with CartPole environment."""
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
             n_epochs = 10
-            n_epoch_cycles = 10
+            steps_per_epoch = 10
             sampler_batch_size = 500
-            num_timesteps = n_epochs * n_epoch_cycles * sampler_batch_size
-            env = TfEnv(gym.make('CartPole-v0'))
-            replay_buffer = SimpleReplayBuffer(env_spec=env.spec,
-                                               size_in_transitions=int(1e4),
-                                               time_horizon=1)
+            num_timesteps = n_epochs * steps_per_epoch * sampler_batch_size
+            env = GarageEnv(gym.make('CartPole-v0'))
+            replay_buffer = PathBuffer(capacity_in_transitions=int(1e4))
             qf = DiscreteMLPQFunction(env_spec=env.spec, hidden_sizes=(64, 64))
             policy = DiscreteQfDerivedPolicy(env_spec=env.spec, qf=qf)
-            epilson_greedy_strategy = EpsilonGreedyStrategy(
+            epilson_greedy_policy = EpsilonGreedyPolicy(
                 env_spec=env.spec,
+                policy=policy,
                 total_timesteps=num_timesteps,
                 max_epsilon=1.0,
                 min_epsilon=0.02,
@@ -170,7 +163,7 @@ class TestDQN(TfGraphTestCase):
             algo = DQN(env_spec=env.spec,
                        policy=policy,
                        qf=qf,
-                       exploration_strategy=epilson_greedy_strategy,
+                       exploration_policy=epilson_greedy_policy,
                        replay_buffer=replay_buffer,
                        qf_lr=1e-4,
                        discount=1.0,
@@ -178,7 +171,7 @@ class TestDQN(TfGraphTestCase):
                        double_q=False,
                        n_train_steps=500,
                        grad_norm_clipping=5.0,
-                       n_epoch_cycles=n_epoch_cycles,
+                       steps_per_epoch=steps_per_epoch,
                        target_network_update_freq=1,
                        buffer_batch_size=32)
             runner.setup(algo, env)
