@@ -1,10 +1,9 @@
 import pytest
 
-from garage.experiment import deterministic
+from garage.envs import GarageEnv
+from garage.experiment import LocalTFRunner
 from garage.np.baselines import LinearFeatureBaseline
 from garage.tf.algos import ERWR
-from garage.tf.envs import TfEnv
-from garage.tf.experiment import LocalTFRunner
 from garage.tf.policies import CategoricalMLPPolicy
 from tests.fixtures import snapshot_config, TfGraphTestCase
 
@@ -15,8 +14,8 @@ class TestERWR(TfGraphTestCase):
     def test_erwr_cartpole(self):
         """Test ERWR with Cartpole-v1 environment."""
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
-            deterministic.set_seed(1)
-            env = TfEnv(env_name='CartPole-v1')
+            env = GarageEnv(env_name='CartPole-v1')
+
             policy = CategoricalMLPPolicy(name='policy',
                                           env_spec=env.spec,
                                           hidden_sizes=(32, 32))
