@@ -6,13 +6,12 @@ import gym
 import pytest
 import tensorflow as tf
 
-from garage.envs import normalize
+from garage.envs import GarageEnv, normalize
+from garage.experiment import LocalTFRunner
 from garage.np.baselines import LinearFeatureBaseline
 from garage.tf.algos import PPO
 from garage.tf.baselines import ContinuousMLPBaseline
 from garage.tf.baselines import GaussianMLPBaseline
-from garage.tf.envs import TfEnv
-from garage.tf.experiment import LocalTFRunner
 from garage.tf.policies import CategoricalMLPPolicy
 from garage.tf.policies import GaussianGRUPolicy
 from garage.tf.policies import GaussianLSTMPolicy
@@ -25,7 +24,7 @@ class TestPPO(TfGraphTestCase):
 
     def setup_method(self):
         super().setup_method()
-        self.env = TfEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
+        self.env = GarageEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
         self.policy = GaussianMLPPolicy(
             env_spec=self.env.spec,
             hidden_sizes=(64, 64),
@@ -39,8 +38,7 @@ class TestPPO(TfGraphTestCase):
             regressor_args=dict(hidden_sizes=(32, 32)),
         )
 
-    # large marker removed to balance test jobs
-    # @pytest.mark.large
+    @pytest.mark.mujoco
     def test_ppo_pendulum(self):
         """Test PPO with Pendulum environment."""
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
@@ -55,8 +53,7 @@ class TestPPO(TfGraphTestCase):
             last_avg_ret = runner.train(n_epochs=10, batch_size=2048)
             assert last_avg_ret > 35
 
-    # large marker removed to balance test jobs
-    # @pytest.mark.large
+    @pytest.mark.mujoco
     def test_ppo_with_maximum_entropy(self):
         """Test PPO with maxium entropy method."""
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
@@ -75,8 +72,7 @@ class TestPPO(TfGraphTestCase):
             last_avg_ret = runner.train(n_epochs=10, batch_size=2048)
             assert last_avg_ret > 35
 
-    # large marker removed to balance test jobs
-    # @pytest.mark.large
+    @pytest.mark.mujoco
     def test_ppo_with_neg_log_likeli_entropy_estimation_and_max(self):
         """
         Test PPO with negative log likelihood entropy estimation and max
@@ -99,8 +95,7 @@ class TestPPO(TfGraphTestCase):
             last_avg_ret = runner.train(n_epochs=10, batch_size=2048)
             assert last_avg_ret > 35
 
-    # large marker removed to balance test jobs
-    # @pytest.mark.large
+    @pytest.mark.mujoco
     def test_ppo_with_neg_log_likeli_entropy_estimation_and_regularized(self):
         """
         Test PPO with negative log likelihood entropy estimation and
@@ -123,8 +118,7 @@ class TestPPO(TfGraphTestCase):
             last_avg_ret = runner.train(n_epochs=10, batch_size=2048)
             assert last_avg_ret > 35
 
-    # large marker removed to balance test jobs
-    # @pytest.mark.large
+    @pytest.mark.mujoco
     def test_ppo_with_regularized_entropy(self):
         """Test PPO with regularized entropy method."""
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
@@ -144,10 +138,11 @@ class TestPPO(TfGraphTestCase):
             last_avg_ret = runner.train(n_epochs=10, batch_size=2048)
             assert last_avg_ret > 35
 
+    @pytest.mark.mujoco
     def test_ppo_pendulum_flatten_input(self):
         """Test PPO with CartPole to test observation flattening."""
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
-            env = TfEnv(
+            env = GarageEnv(
                 normalize(ReshapeObservation(gym.make('CartPole-v1'), (2, 2))))
             policy = CategoricalMLPPolicy(
                 env_spec=env.spec,
@@ -165,7 +160,7 @@ class TestPPO(TfGraphTestCase):
                        optimizer_args=dict(
                            batch_size=32,
                            max_epochs=10,
-                           tf_optimizer_args=dict(learning_rate=1e-3),
+                           learning_rate=1e-3,
                        ))
             runner.setup(algo, env)
             last_avg_ret = runner.train(n_epochs=10, batch_size=2048)
@@ -182,7 +177,7 @@ class TestPPOContinuousBaseline(TfGraphTestCase):
     def test_ppo_pendulum_continuous_baseline(self):
         """Test PPO with Pendulum environment."""
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
-            env = TfEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
+            env = GarageEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
             policy = GaussianMLPPolicy(
                 env_spec=env.spec,
                 hidden_sizes=(64, 64),
@@ -216,11 +211,11 @@ class TestPPOContinuousBaseline(TfGraphTestCase):
 
             env.close()
 
-    @pytest.mark.large
+    @pytest.mark.mujoco_long
     def test_ppo_pendulum_recurrent_continuous_baseline(self):
         """Test PPO with Pendulum environment and recurrent policy."""
         with LocalTFRunner(snapshot_config) as runner:
-            env = TfEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
+            env = GarageEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
             policy = GaussianLSTMPolicy(env_spec=env.spec, )
             baseline = ContinuousMLPBaseline(
                 env_spec=env.spec,
@@ -252,11 +247,11 @@ class TestPPOContinuousBaseline(TfGraphTestCase):
 
 class TestPPOPendulumLSTM(TfGraphTestCase):
 
-    @pytest.mark.large
+    @pytest.mark.mujoco_long
     def test_ppo_pendulum_lstm(self):
         """Test PPO with Pendulum environment and recurrent policy."""
         with LocalTFRunner(snapshot_config) as runner:
-            env = TfEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
+            env = GarageEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
             lstm_policy = GaussianLSTMPolicy(env_spec=env.spec)
             baseline = GaussianMLPBaseline(
                 env_spec=env.spec,
@@ -286,11 +281,11 @@ class TestPPOPendulumLSTM(TfGraphTestCase):
 
 class TestPPOPendulumGRU(TfGraphTestCase):
 
-    @pytest.mark.large
+    @pytest.mark.mujoco_long
     def test_ppo_pendulum_gru(self):
         """Test PPO with Pendulum environment and recurrent policy."""
         with LocalTFRunner(snapshot_config) as runner:
-            env = TfEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
+            env = GarageEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
             gru_policy = GaussianGRUPolicy(env_spec=env.spec)
             baseline = GaussianMLPBaseline(
                 env_spec=env.spec,
