@@ -6,15 +6,14 @@ import gym
 import pytest
 import tensorflow as tf
 
-from garage.envs import normalize
+from garage.envs import GarageEnv, normalize
 from garage.experiment import deterministic
+from garage.experiment import LocalTFRunner
 from garage.experiment import snapshotter
 from garage.np.baselines import LinearFeatureBaseline
 from garage.tf.algos import TRPO
 from garage.tf.baselines import GaussianCNNBaseline
 from garage.tf.baselines import GaussianMLPBaseline
-from garage.tf.envs import TfEnv
-from garage.tf.experiment import LocalTFRunner
 from garage.tf.optimizers import FiniteDifferenceHvp
 from garage.tf.policies import CategoricalCNNPolicy
 from garage.tf.policies import CategoricalGRUPolicy
@@ -27,7 +26,7 @@ class TestTRPO(TfGraphTestCase):
 
     def setup_method(self):
         super().setup_method()
-        self.env = TfEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
+        self.env = GarageEnv(normalize(gym.make('InvertedDoublePendulum-v2')))
         self.policy = GaussianMLPPolicy(
             env_spec=self.env.spec,
             hidden_sizes=(64, 64),
@@ -39,7 +38,7 @@ class TestTRPO(TfGraphTestCase):
             regressor_args=dict(hidden_sizes=(32, 32)),
         )
 
-    @pytest.mark.large
+    @pytest.mark.mujoco_long
     def test_trpo_pendulum(self):
         """Test TRPO with Pendulum environment."""
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
@@ -52,8 +51,9 @@ class TestTRPO(TfGraphTestCase):
                         policy_ent_coeff=0.0)
             runner.setup(algo, self.env)
             last_avg_ret = runner.train(n_epochs=10, batch_size=2048)
-            assert last_avg_ret > 50
+            assert last_avg_ret > 40
 
+    @pytest.mark.mujoco
     def test_trpo_unknown_kl_constraint(self):
         """Test TRPO with unkown KL constraints."""
         with pytest.raises(ValueError, match='Invalid kl_constraint'):
@@ -68,7 +68,7 @@ class TestTRPO(TfGraphTestCase):
                 kl_constraint='random kl_constraint',
             )
 
-    @pytest.mark.large
+    @pytest.mark.mujoco_long
     def test_trpo_soft_kl_constraint(self):
         """Test TRPO with unkown KL constraints."""
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
@@ -84,10 +84,10 @@ class TestTRPO(TfGraphTestCase):
             last_avg_ret = runner.train(n_epochs=10, batch_size=2048)
             assert last_avg_ret > 45
 
-    @pytest.mark.large
+    @pytest.mark.mujoco_long
     def test_trpo_lstm_cartpole(self):
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
-            env = TfEnv(normalize(gym.make('CartPole-v1')))
+            env = GarageEnv(normalize(gym.make('CartPole-v1')))
 
             policy = CategoricalLSTMPolicy(name='policy', env_spec=env.spec)
 
@@ -109,11 +109,11 @@ class TestTRPO(TfGraphTestCase):
 
             env.close()
 
-    @pytest.mark.large
+    @pytest.mark.mujoco_long
     def test_trpo_gru_cartpole(self):
         deterministic.set_seed(2)
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
-            env = TfEnv(normalize(gym.make('CartPole-v1')))
+            env = GarageEnv(normalize(gym.make('CartPole-v1')))
 
             policy = CategoricalGRUPolicy(name='policy', env_spec=env.spec)
 
@@ -130,7 +130,7 @@ class TestTRPO(TfGraphTestCase):
 
             runner.setup(algo, env)
             last_avg_ret = runner.train(n_epochs=10, batch_size=2048)
-            assert last_avg_ret > 40
+            assert last_avg_ret > 80
 
             env.close()
 
@@ -144,23 +144,21 @@ class TestTRPOCNNCubeCrash(TfGraphTestCase):
     @pytest.mark.large
     def test_trpo_cnn_cubecrash(self):
         with LocalTFRunner(snapshot_config, sess=self.sess) as runner:
-            env = TfEnv(normalize(gym.make('CubeCrash-v0')))
+            env = GarageEnv(normalize(gym.make('CubeCrash-v0')))
 
             policy = CategoricalCNNPolicy(env_spec=env.spec,
-                                          conv_filters=(32, 64),
-                                          conv_filter_sizes=(8, 4),
-                                          conv_strides=(4, 2),
-                                          conv_pad='VALID',
+                                          filters=((32, (8, 8)), (64, (4, 4))),
+                                          strides=(4, 2),
+                                          padding='VALID',
                                           hidden_sizes=(32, 32))
 
-            baseline = GaussianCNNBaseline(env_spec=env.spec,
-                                           regressor_args=dict(
-                                               num_filters=(32, 64),
-                                               filter_dims=(8, 4),
-                                               strides=(4, 2),
-                                               padding='VALID',
-                                               hidden_sizes=(32, 32),
-                                               use_trust_region=True))
+            baseline = GaussianCNNBaseline(
+                env_spec=env.spec,
+                regressor_args=dict(filters=((32, (8, 8)), (64, (4, 4))),
+                                    strides=(4, 2),
+                                    padding='VALID',
+                                    hidden_sizes=(32, 32),
+                                    use_trust_region=True))
 
             algo = TRPO(env_spec=env.spec,
                         policy=policy,
@@ -174,6 +172,6 @@ class TestTRPOCNNCubeCrash(TfGraphTestCase):
 
             runner.setup(algo, env)
             last_avg_ret = runner.train(n_epochs=10, batch_size=2048)
-            assert last_avg_ret > -0.9
+            assert last_avg_ret > -1.5
 
             env.close()
